@@ -79,7 +79,7 @@ nil means use indentation.")
 (defvar mail-indentation-spaces 3
   "*Number of spaces to insert at the beginning of each cited line.
 Used by `mail-yank-original' via `mail-yank-cite'.")
-(defvar mail-yank-hooks '(mail-indent-citation)
+(defvar mail-yank-hooks nil
   "Obsolete hook for modifying a citation just inserted in the mail buffer.
 Each hook function can find the citation between (point) and (mark t).
 And each hook function should leave point and mark around the citation
@@ -248,6 +248,9 @@ C-c C-v  mail-sent-via (add a sent-via field for each To or CC)."
 (define-key mail-mode-map [menu-bar mail signature]
   '("Insert Signature" . mail-signature))
 
+(define-key mail-mode-map [menu-bar mail cancel]
+  '("Cancel" . mail-dont-send))
+
 (define-key mail-mode-map [menu-bar mail send-stay]
   '("Send, Keep Editing" . mail-send))
 
@@ -283,15 +286,29 @@ C-c C-v  mail-sent-via (add a sent-via field for each To or CC)."
 Prefix arg means don't delete this window."
   (interactive "P")
   (mail-send)
+  (mail-bury arg))
+
+(defun mail-dont-send (arg)
+  "Don't send the message you have been editing.
+Prefix arg means don't delete this window."
+  (interactive "P")
+  (mail-bury arg))
+
+(defun mail-bury (arg)
+  "Bury this mail buffer."
   (let ((newbuf (other-buffer (current-buffer))))
     (bury-buffer (current-buffer))
-    (if (and (not arg)
-	     (not (one-window-p))
-	     (save-excursion
-	       (set-buffer (window-buffer (next-window (selected-window) 'not)))
-	       (eq major-mode 'rmail-mode)))
-	(delete-window)
-      (switch-to-buffer newbuf))))
+    (if (and (fboundp 'frame-parameters)
+	     (cdr (assq 'dedicated (frame-parameters)))
+	     (not (null (delq (selected-frame) (visible-frame-list)))))
+	(delete-frame (selected-frame))
+      (if (and (not arg)
+	       (not (one-window-p))
+	       (save-excursion
+		 (set-buffer (window-buffer (next-window (selected-window) 'not)))
+		 (eq major-mode 'rmail-mode)))
+	  (delete-window)
+	(switch-to-buffer newbuf)))))
 
 (defun mail-send ()
   "Send the message in the current buffer.
@@ -422,7 +439,6 @@ the user from the mailer."
   (let (fcc-list
 	(rmailbuf (current-buffer))
 	(time (current-time))
-	timezone
 	(tembuf (generate-new-buffer " rmail output"))
 	(case-fold-search t))
     (save-excursion
@@ -436,13 +452,6 @@ the user from the mailer."
 			     fcc-list))
 	(delete-region (match-beginning 0)
 		       (progn (forward-line 1) (point))))
-      (let* ((foo (current-time-zone time))
-	     (offset (if (car foo) (/ (car foo) 60) 0))
-	     (abs (abs offset)))
-	(setq timezone (format "%s%02d%02d"
-			       (if (< offset 0) "-" "+")
-			       (/ abs 60)
-			       (% abs 60))))
       (set-buffer tembuf)
       (erase-buffer)
       ;; This initial newline is written out if the fcc file already exists.
@@ -451,7 +460,8 @@ the user from the mailer."
       ;; Insert the time zone before the year.
       (forward-char -1)
       (forward-word -1)
-      (insert timezone " ")
+      (require 'mail-utils)
+      (insert (mail-rfc822-time-zone time) " ")
       (goto-char (point-max))
       (insert-buffer-substring rmailbuf)
       ;; Make sure messages are separated.
@@ -493,7 +503,7 @@ the user from the mailer."
 				(narrow-to-region (point-max) (point-max))
 				(insert "\C-l\n0, unseen,,\n*** EOOH ***\n"
 					"From: " (user-login-name) "\n"
-					"Date: " (current-time-string) "\n")
+					"Date: " (mail-rfc822-date) "\n")
 				(insert-buffer-substring curbuf beg2 end)
 				(insert "\n\C-_")
 				(goto-char (point-min))
@@ -668,7 +678,9 @@ and don't delete any header fields."
 					   mail-indentation-spaces)))
 	    (if mail-citation-hook
 		(run-hooks 'mail-citation-hook)
-	      (run-hooks 'mail-yank-hooks))))
+	      (if mail-yank-hooks
+		  (run-hooks 'mail-yank-hooks)
+		(mail-indent-citation)))))
 	;; This is like exchange-point-and-mark, but doesn't activate the mark.
 	;; It is cleaner to avoid activation, even though the command
 	;; loop would deactivate the mark because we inserted text.
@@ -821,16 +833,11 @@ The seventh argument ACTIONS is a list of actions to take
     (pop-to-buffer "*mail*"))
   (mail noerase to subject in-reply-to cc replybuffer sendactions))
 
-
-;;;###autoload
-(define-key ctl-x-map "m" 'mail)
-
-;;;###autoload
-(define-key ctl-x-4-map "m" 'mail-other-window)
-
-;;;###autoload
-(define-key ctl-x-5-map "m" 'mail-other-frame)
-
+;;; Do not execute these when sendmail.el is loaded,
+;;; only in loaddefs.el.
+;;;###autoload (define-key ctl-x-map "m" 'mail)
+;;;###autoload (define-key ctl-x-4-map "m" 'mail-other-window)
+;;;###autoload (define-key ctl-x-5-map "m" 'mail-other-frame)
 
 ;;; Do not add anything but external entries on this page.
 
